import java.util.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

/**
 * Animation2D uses classes developed in last week's Transform2DTest project to
 * confirm the applicability of Matrix-defined transformations to simple
 * animations. Note: <b>Add Path</b> to <b>Transform2DTest</b> folder in
 * <b>Classes</b> Tab of your <b>JDK Profile.</b> 
 * 
 * @author C. D'Arcy
 */

public class Animation2D extends Canvas {

	/**
	 * The width of the Canvas
	 */
	public static final int WIDTH = 305;

	/**
	 * The height of the Canvas
	 */
	public static final int HEIGHT = 305;

	/**
	 * SPEED controls the delay of the utility Thread
	 */

	public static final int SPEED = 100;

	/**
	 * Specialized class to support double buffering
	 */
	public BufferStrategy strategy;

	/**
	 * Cartesian parameters: domain, range, minX, maxX, minY, maxY
	 */
	public static double maxX = 10.0;
	public static double minX = -10.0;
	public static double maxY = 10.0;
	public static double minY = -10.0;

	/**
	 * assists in converting cartesian coordinates to screen coordinates
	 */
	public static double domain = maxX - minX;

	/**
	 * assists in converting Cartesian coordinates to Screen coordinates
	 */
	public static double range = maxY - minY;

	// Animation details---------------------
	private static double delta = Math.PI / 180;
	private static double angle = -delta;
	private static double transX = 0.1;
	private static double transY = -0.1;

	/**
	 * an instance of the <b>Sprite</b> class to be animated
	 */

	private static Sprite frontWheel;
	private static Sprite backWheel;
	private static Sprite building;
	private static Transform2D s1A, s1A1, s1A2, s1A3, s2A, s2A1, s2A2, s2A3,
	bInitialTrans, bTrans;

	/**
	 * Single constructor (supplied)
	 */
	public Animation2D() {
		JFrame frame = new JFrame("2D Animation ");
		JPanel panel = (JPanel) frame.getContentPane();

		setBounds(0, 0, WIDTH, HEIGHT);

		panel.setPreferredSize(new Dimension(WIDTH, HEIGHT));
		panel.setLayout(null);
		panel.add(this);
		frame.setBounds(0, 0, WIDTH, HEIGHT);
		frame.setVisible(true);
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		frame.setResizable(false);
		createBufferStrategy(2);
		strategy = getBufferStrategy();
		requestFocus();

	}

	/**
	 * updateScene prepares and applies the matrix transform(s) to the Sprite
	 * object
	 */

	public void updateScene() {
		// some interesting transformations here...
		frontWheel.transform(s1A3);
		backWheel.transform(s2A3);
		building.transform(bTrans);
		Point2DList list = building.getData();
		double x =list.getPoints().get(0).getCoordinate(0);
		if (x<-12){
			Transform2D reset = new Transform2D();
			reset.setTranslation(new Point2D(24,0));
			building.transform(reset);
		}

	}

	/**
	 * Converts Cartesian x-coordinate to relative Screen x-coordinate
	 */
	public int scrX(double x) {
		int sx = 0;

		// remember the Mandelbrot assignment?
		sx = (int) ((x - minX) * WIDTH / domain);

		return sx;
	}

	/**
	 * Converts Cartesian y-coordinate to relative Screen y-coordinate
	 */
	public int scrY(double y) {
		int sy = 0;

		// remember the Mandelbrot assignment?
		sy = (int) ((maxY - y) * HEIGHT / range);
		return sy;
	}

	/**
	 * Creates a BufferedImage object, obtains its Graphics2D drawing context,
	 * and goes to work drawing to the image. Calls sprite's getData() method to
	 * obtain the set of points (vertices) transformed, drawn and connected.
	 * Hint. Consider copying the points to a java.awt.Polygon object, and using
	 * Graphics2D's drawPolygon(poly) or fillPolygon(poly) method. Once
	 * rendered, obtain the visible drawing context from the strategy object,
	 * copying the BufferedImage to the visible canvas with the drawImage()
	 * method. Finally, strategy.show().
	 */
	public void paintPanel() {
		BufferedImage image = new BufferedImage(WIDTH, HEIGHT,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D g2D = (Graphics2D) image.getGraphics();

		// Setup the blank canvas
		g2D.setColor(Color.black);
		g2D.fillRect(0, 0, getWidth(), getHeight());

		// your stuff here
		ArrayList<Point2D> list = building.getData().getPoints();
		Polygon poly = new Polygon();

		for (Point2D p : list) {
			poly.addPoint(scrX(p.getCoordinate(0)), scrY(p.getCoordinate(1)));
		}
		g2D.setColor(Color.GRAY);
		g2D.fillPolygon(poly);


		
		
		list = frontWheel.getData().getPoints();
		poly = new Polygon();

		for (Point2D p : list) {
			poly.addPoint(scrX(p.getCoordinate(0)), scrY(p.getCoordinate(1)));
		}
		g2D.setColor(Color.YELLOW);
		g2D.fillPolygon(poly);

		list = backWheel.getData().getPoints();
		poly = new Polygon();
		for (Point2D p : list) {
			poly.addPoint(scrX(p.getCoordinate(0)), scrY(p.getCoordinate(1)));
		}
	//	g2D.setColor(Color.YELLOW);
		g2D.fillPolygon(poly);

		
		
		Graphics g = strategy.getDrawGraphics();

		g.drawImage(image, 0, 0, this);
		strategy.show();
	}

	/**
	 * Sequencer to organize calls and prove an animation speed.
	 */
	public void run() {
		while (this.isVisible()) {
			updateScene();
			paintPanel();
			try {
				Thread.sleep(SPEED);
			} catch (InterruptedException e) {
			}
		}

	}

	/**
	 * Instantiates an animation object,a sprite with some meaningful set of
	 * points and then calls the run method.
	 */
	public static void main(String[] args) {
		/*
		 * tr1A = new Transform2D(); tr1B = new Transform2D(); tr2A = new
		 * Transform2D(); tr2B = new Transform2D();
		 * 
		 * tr1A.setRotation(5 * Math.PI / 180); tr1B.setScale(1.05); tr1A =
		 * tr1A.mul(tr1B);
		 * 
		 * tr2A.setRotation(-Math.PI / 180); tr2B.setScale(0.95); tr2A =
		 * tr2A.mul(tr2B);
		 * 
		 * Animation2D animation = new Animation2D();
		 * 
		 * spriteOne = new Sprite(); spriteOne.createData(); spriteTwo = new
		 * Sprite(); spriteTwo.createData();
		 */
		Animation2D animation = new Animation2D();
		frontWheel = new Sprite();
		backWheel = new Sprite();
		building = new Sprite();

		frontWheel.createData(8);
		s1A = new Transform2D();
		s1A1 = new Transform2D();
		s1A2 = new Transform2D();
		s1A3 = new Transform2D();
		s1A.setTranslation(new Point2D(3, 0));
		frontWheel.transform(s1A);
		s1A1.setTranslation(new Point2D(-3, 0));
		s1A2.setRotation(-Math.PI / 12);
		s1A3 = s1A.mul(s1A2.mul(s1A1));

		backWheel.createData(8);
		backWheel.transform(s1A1);
		s2A3 = s1A1.mul(s1A2.mul(s1A));

		building.createData(4);
		bInitialTrans = new Transform2D();
		bInitialTrans.setTranslation(new Point2D(11,2));
		Transform2D rot45 = new Transform2D();
		rot45.setRotation(Math.PI/4);
		Transform2D scale = new Transform2D();
		scale.setVerticalScale(4);
		
		building.transform(bInitialTrans.mul(scale.mul(rot45)));

		bTrans = new Transform2D();
		bTrans.setTranslation(new Point2D(-0.1,0));

		
		animation.run();
	}

}
